@@ -38,6 +38,7 @@
 
 #undef MEASURE_TIME
 
+#include "sylmain.h"
 #include "folder.h"
 #include "mh.h"
 #include "procmsg.h"
@@ -459,18 +460,16 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 		if (first_ == 0 || first_ > dest->last_num + 1)
 			first_ = dest->last_num + 1;
 
-#ifdef G_OS_UNIX
-		if (link(fileinfo->file, destfile) < 0) {
-#endif
+		if (syl_link(fileinfo->file, destfile) < 0) {
 			if (copy_file(fileinfo->file, destfile, TRUE) < 0) {
 				g_warning(_("can't copy message %s to %s\n"),
 					  fileinfo->file, destfile);
 				g_free(destfile);
 				return -1;
 			}
-#ifdef G_OS_UNIX
 		}
-#endif
+
+		g_signal_emit_by_name(syl_app_get(), "add-msg", dest, destfile, dest->last_num + 1);
 
 		g_free(destfile);
 		dest->last_num++;
@@ -555,12 +554,16 @@ static gint mh_do_move_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 		if (!destfile) break;
 		srcfile = procmsg_get_message_file(msginfo);
 
+		g_signal_emit_by_name(syl_app_get(), "remove-msg", src, srcfile, msginfo->msgnum);
+
 		if (move_file(srcfile, destfile, FALSE) < 0) {
 			g_free(srcfile);
 			g_free(destfile);
 			break;
 		}
 
+		g_signal_emit_by_name(syl_app_get(), "add-msg", dest, destfile, dest->last_num + 1);
+
 		g_free(srcfile);
 		g_free(destfile);
 		src->total--;
@@ -683,6 +686,8 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 			break;
 		}
 
+		g_signal_emit_by_name(syl_app_get(), "add-msg", dest, destfile, dest->last_num + 1);
+
 		g_free(srcfile);
 		g_free(destfile);
 		dest->last_num++;
@@ -715,6 +720,8 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 	file = mh_fetch_msg(folder, item, msginfo->msgnum);
 	g_return_val_if_fail(file != NULL, -1);
 
+	g_signal_emit_by_name(syl_app_get(), "remove-msg", item, file, msginfo->msgnum);
+
 	if (g_unlink(file) < 0) {
 		FILE_OP_ERROR(file, "unlink");
 		g_free(file);
@@ -746,6 +753,7 @@ static gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 
 	path = folder_item_get_path(item);
 	g_return_val_if_fail(path != NULL, -1);
+	g_signal_emit_by_name(syl_app_get(), "remove-all-msg", item);
 	val = remove_all_numbered_files(path);
 	g_free(path);
 	if (val == 0) {
@@ -1092,6 +1100,7 @@ static gint mh_move_folder_real(Folder *folder, FolderItem *item,
 	gchar *name_;
 	gchar *utf8_name;
 	gchar *paths[2];
+	gchar *old_id, *new_id;
 
 	g_return_val_if_fail(folder != NULL, -1);
 	g_return_val_if_fail(item != NULL, -1);
@@ -1167,6 +1176,8 @@ static gint mh_move_folder_real(Folder *folder, FolderItem *item,
 	g_free(oldpath);
 	g_free(newpath);
 
+	old_id = folder_item_get_identifier(item);
+
 	if (new_parent) {
 		g_node_unlink(item->node);
 		g_node_append(new_parent->node, item->node);
@@ -1200,6 +1211,12 @@ static gint mh_move_folder_real(Folder *folder, FolderItem *item,
 	g_free(paths[0]);
 	g_free(paths[1]);
 
+	new_id = folder_item_get_identifier(item);
+	g_signal_emit_by_name(syl_app_get(), "move-folder", item, old_id,
+			      new_id);
+	g_free(new_id);
+	g_free(old_id);
+
 	return 0;
 }
 
@@ -1231,6 +1248,7 @@ static gint mh_remove_folder(Folder *folder, FolderItem *item)
 	}
 
 	g_free(path);
+	g_signal_emit_by_name(syl_app_get(), "remove-folder", item);
 	folder_item_remove(item);
 	return 0;
 }
